#!/bin/bash
# USAGE: sh ~/<path_to_file>/Supplemental_file_1.sh
# Christopher J. Tracy
# June 26, 2023
#
## The purpose of this script is to take a directory of split paired-end fastq.gz files from an RNA-seq experiment, perform quality assessment on them including trimming and quality filtering, then align them by coordinate against a previously indexed reference genome (Gencode), perform a final round of quality assessment on the alignments, and then output a featurecounts matrix for further processing in Vim.
#
# Step 0.  Defines reference directories and variables.
# Step 1.  Run FastQC on .fastq.gz files within a directory; run MultiQC on the FastQC output directory.
# Step 2.  Run CutAdapt on the .fastq.gz files from Step 1.
# Step 3.  Run FastQC on the .fastq.gz files from Step 2; run MultiQC on the FastQC output directory.
# Step 4.  Run STAR on the .fastq.gz files from Step 2; run Samtools index on the Aligned.sortedByCoord.out.bam files from STAR.
# Step 6.  Run Qualimap on the Aligned.sortedByCoord.out.bam files from Step 4.  
# Step 7.  Run Subread featureCounts on the Aligned.sortedByCoord.out.bam files from Step 4.
# Step 8.  Run MultiQC on the output directories of Steps 4, 5, and 6.
# Step 9.  Run MultiQC on the output directories of Steps 1, 2, 3, 4, 5, and 6.
#
## This script makes use of the following software:
# FastQC-0.12.1
# CutAdapt-4.3; installed through Conda, uses the "cutadapt" environment
# STAR-2.7.10.b
# Samtools-1.17
# QualiMap v.2.2.2d; installed through Conda, uses the "qualimap" environment
# Subread-2.0.4
# MultiQC-1.14; installed through Conda, uses the "multiqc" environment
# 
## Note:  All file paths are specific to a user's computer and will need to be tailored to each machine.
#
## Note:  This pipeline uses the human Gencode implementation of the Ensembl reference genome, GRCh38.p13.43, indexed with a 149 bp sjdb overhang.

## Captures the terminal output within the "terminal_log.txt" file.
exec > /<path_to_file>/paired/terminal_log.txt 2>&1
set -x
## Note:  In order to visualize the progress of this file being written to, in another terminal window, run the following command:
## $ tail -f /<path_to_file>/paired/terminal_log.txt
## Note:  Once the script is finished running in the other terminal window, use the keyboard shortcut ctrl+C to stop the tail command.

echo "This alignment pipeline is using the script Supplemental_file_1.sh."
date

## Step 0 Setup ----------------------------------------------------------------------------------------------------------------
# Defines the genome directories for use with STAR.
genome_gencode=/<path_to_file>/reference_data/gencode_GRCh38.p13.43_150bp
# Defines the gtf variables for use with Qualimap and featureCounts.
gtf_gencode=/<path_to_file>/reference_data/gencode_GRCh38.p13.43_150bp/gencode.v43.primary_assembly.annotation.gtf

## Step 1 Pre-Trimming QC ------------------------------------------------------------------------------------------------------
# Runs FastQC on the raw split paired-end fastq.gz files and outputs a FastQC report for each file, then runs MultiQC to collate the reports.

## Step 1.1 FastQC -------------------------------------------------------------------------------------------------------------
# A FastQC manual can be found at the following webpage:  https://dnacore.missouri.edu/PDF/FastQC_Manual.pdf
# Creates the directory for the FastQC reports and then changes directories to /<path_to_file>/paired/raw_files.
mkdir -p /<path_to_file>/paired/results/fastqc/before_trimming
cd /<path_to_file>/paired/raw_files
# Sets the output folder for the FastQC command.
fastqc_out=/<path_to_file>/paired/results/fastqc/before_trimming
# Runs the FastQC command using 60 threads in parallel on all the fastq.gz files in the raw_files directory.
find . -name "*.fastq.gz" | xargs fastqc -t 60 -o $fastqc_out

## Step 1.2 MultiQC ------------------------------------------------------------------------------------------------------------
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<path_to_file>/paired/results/fastqc/before_trimming.
cd /<path_to_file>/paired/results/fastqc/before_trimming
# Runs multiqc in the current directory.
multiqc -n before_trimming_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 2 Adapter Removal and Quality Filtering and Trimming -------------------------------------------------------------------
# Runs CutAdapt to trim and quality filter the raw paired-end fastq.gz files and outputs a report for each file in addition to the trimmed and filtered fastq files, then runs MultiQC to collate the reports.

## Step 2.1 Cutadapt -----------------------------------------------------------------------------------------------------------
# The CutAdapt manual can be found at the following webpage:  https://cutadapt.readthedocs.io/en/stable/guide.html
## Note:  This process effectively duplicates the contents of the raw_files directory, so it is imperative to ensure that there is sufficient space on the hard drive for all of the raw and output files (at least double the size of the raw_files directory).
# Activates the "cutadapt" Conda environment which CutAdapt runs within.
eval "$(conda shell.bash hook)"
conda activate cutadapt
# Creates the directory for the output of CutAdapt.  
mkdir -p /<path_to_file>/paired/results/cutadapt/
cd /<path_to_file>/paired/raw_files/
# Establishes the naming parameters for the for loop.
f1=$(ls *_1.fastq.gz | sed 's/_1.fastq.gz//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the fastq.gz files within the raw_files directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename"
cutadapt_out=/<path_to_file>/paired/results/cutadapt
echo "Starting trimming and filtering for $samplename using CutAdapt."
# Runs the CutAdapt command in multi-core support mode with 60 cores.  Removes any reads shorter than 50 bp and trims any bases with a PHRED score of lower than 20.  This will result in some of the individual reads no longer being all the same length (for example, prior to trimming, reads could all be 150 bp; after trimming, depending on how many bases are trimmed, reads could range from 147-150 bp).
cutadapt -j 60 \
-a AGATCGGAAGAGCACACGTCTGAACTCCAGTCA \
-A AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGT \
--nextseq-trim=20 \
-m 50 \
-o $cutadapt_out/${fn}_trimmed_1.fastq.gz \
-p $cutadapt_out/${fn}_trimmed_2.fastq.gz \
${fn}_1.fastq.gz ${fn}_2.fastq.gz \
1> /<path_to_file>1/paired/results/cutadapt/${fn}_report.txt
done
# Deactivates the "cutadapt" Conda environment.
conda deactivate

## Step 2.2 MultiQC ------------------------------------------------------------------------------------------------------------
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<path_to_file>/paired/results/cutadapt.
cd /<path_to_file>/paired/results/cutadapt
# Runs multiqc in the current directory.
multiqc -n cutadapt_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 3 Post-Trimming QC -----------------------------------------------------------------------------------------------------
# Runs FastQC on the trimmed and quality-filtered fastq.gz files from Step 2 and outputs a FastQC report for each file, then runs MultiQC to collate the reports.

## Step 3.1 FastQC -------------------------------------------------------------------------------------------------------------
# Creates the directory for the FastQC reports and then changes directories to /<path_to_file>/paired/results/cutadapt.
mkdir -p /<path_to_file>/paired/results/fastqc/after_trimming
cd /<path_to_file>/paired/results/cutadapt
# Sets the output folder for the FastQC command.
fastqc_out=/<path_to_file>/paired/results/fastqc/after_trimming
# Runs the FastQC command using 60 threads in parallel on all the fastq files in the cutadapt directory.
find . -name "*.fastq.gz" | xargs fastqc -t 60 -o $fastqc_out

## Step 3.2 MultiQC ------------------------------------------------------------------------------------------------------------
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<path_to_file>/paired/results/fastqc/after_trimming.
cd /<path_to_file>/paired/results/fastqc/after_trimming
# Runs multiqc in the current directory.
multiqc -n after_trimming_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 4 Alignment and Indexing -----------------------------------------------------------------------------------------------
# Aligns the trimmed and quality-filtered fastq.gz files from Step 2 using STAR with a previously STAR-indexed reference genome, then outputs the alignments into Aligned.sortedByCoord.out.bam files for each fastq.gz.  Then uses Samtools to create a .bai index for each Aligned.sortedByCoord.out.bam file.

## Step 4.1 STAR Alignment -----------------------------------------------------------------------------------------------------
# Modifies the number of temporary files allowed to be open at a single time so that STAR will run properly.
ulimit -Sn 24576
# Creates the directories for the output of STAR.
mkdir -p /<path_to_file>/paired/results/gencode/STAR/
# Changes directories to /<path_to_file>/paired/results/cutadapt and establishes the naming convention for the for loop.
cd /<path_to_file>/paired/results/cutadapt/
f1=$(ls *_trimmed_1.fastq.gz | sed 's/_trimmed_1.fastq.gz//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the fastq.gz files within the cutadapt directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename."
# Defines the sample output directory variables for STAR.
align_out_gencode=/<path_to_file>/paired/results/gencode/STAR/${samplename}_
# Runs STAR in paired-end mode.
# The STAR manual can be found at the following webpage:  https://github.com/alexdobin/STAR/blob/master/doc/STARmanual.pdf
echo "Starting STAR run for paired-end sample set $samplename using gencode reference."
STAR --runThreadN 60 \
--genomeDir $genome_gencode \
--readFilesIn ${fn}_trimmed_1.fastq.gz ${fn}_trimmed_2.fastq.gz \
--readFilesCommand zcat \
--outFileNamePrefix $align_out_gencode \
--outSAMtype BAM SortedByCoordinate \
--outMultimapperOrder Random \
--outSAMattributes Standard \
--outReadsUnmapped Fastx
done

## Step 4.2 BAM Indexing with Samtools -----------------------------------------------------------------------------------------
# Changes directories to /<path_to_file>/paired/results/gencode/STAR.
cd /<path_to_file>/paired/results/gencode/STAR/
echo "Starting indexing of the gencode BAM files using Samtools."
# Establishes the naming conventions for the for loop for Samtools.
f1=$(ls *_Aligned.sortedByCoord.out.bam | sed 's/_Aligned.sortedByCoord.out.bam//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files within the STAR directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename."
echo "Indexing the BAM file for $samplename."
# Runs Samtools index with 60 threads.
# The documentation for Samtools index can be found at the following webpage:  http://www.htslib.org/doc/samtools-index.html
samtools index ${fn}_Aligned.sortedByCoord.out.bam -@ 60
done

## Step 5 Post-Alignment QC (Qualimap) -----------------------------------------------------------------------------------------
# Activates the "qualimap" Conda environment which Qualimap runs within.
# The Qualimap manual can be found at the following webpage:  http://qualimap.conesalab.org/doc_html/index.html
eval "$(conda shell.bash hook)"
conda activate qualimap
# Creates the directories for the output of Qualimap.
mkdir -p /<path_to_file>/paired/results/gencode/qualimap/
# Unsets the Qualimap display so it does not open the GUI.
unset DISPLAY
# Changes directories to /<path_to_file>/paired/results/gencode/STAR and establishes the naming conventions for the for loop.
cd /<path_to_file>/paired/results/gencode/STAR/
echo "Starting Qualimap QC on gencode Aligned.sortedByCoord.out.bam files."
f1=$(ls *_Aligned.sortedByCoord.out.bam | sed 's/_Aligned.sortedByCoord.out.bam//g' | sort -u)
echo $f1
# Establishes the for loop to run through all of the Aligned.sortedByCoord.out.bam files produced by STAR in the STAR directory.
for fn in $f1
do
samplename=`basename ${fn}`
echo "Sample name is $samplename"
mkdir -p /<path_to_file>/paired/results/gencode/qualimap/${fn}
# Runs Qualimap in rnaseq mode with a memory size allotment of 64 GB.
qualimap rnaseq \
-outdir /<path_to_file>/paired/results/gencode/qualimap/${fn} \
-a proportional \
-bam /<path_to_file>/paired/results/gencode/STAR/${fn}_Aligned.sortedByCoord.out.bam \
-p strand-specific-reverse \
-pe \
-s \
-gtf $gtf_gencode \
--java-mem-size=64G
done
# Deactivates the "qualimap" Conda environment.
conda deactivate

## Step 6 Feature Assignment (Subread featureCounts) ---------------------------------------------------------------------------
# Creates a feature counts matrix using the featureCounts tool from Subread from the Aligned.sortByCoord.out.bam files produced by Samtools from the STAR output.
# The Subread manual can be found at the following webpage:  https://subread.sourceforge.net/SubreadUsersGuide.pdf
# Note:  featureCounts requires that you set the strandedness parameter of the reads.  If this is set incorrectly, the rate of successfully assigned alignments will be low. 
# Creates a directory for the output of Subread and then changes directories to /<path_to_file>/paired/results/gencode/STAR.
mkdir -p /<path_to_file>/paired/results/gencode/counts/
cd /<path_to_file>/paired/results/gencode/STAR/
# Runs featureCounts in paired-end mode, reverse strand, with 32 threads.
echo "Running Subread featureCounts on gencode Aligned.sortedByCoord.out.bam files."
featureCounts -T 32 -s 2 -p -C -d 50 --verbose --countReadPairs -t exon -g gene_id \
-a $gtf_gencode \
-o /<path_to_file>/paired/results/gencode/counts/gencode_featurecounts.txt \
/<path_to_file>/paired/results/gencode/STAR/*_Aligned.sortedByCoord.out.bam

## Step 7 Alignment Summary (MultiQC) ------------------------------------------------------------------------------------------
# Runs MultiQC on all of the alignment and downstream files generated throughout the pipeline and collates them into a single quality assessment report.
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<path_to_file>/paired/results/gencode.
cd /<path_to_file>/paired/results/gencode/
# Runs multiqc in the current directory.
multiqc -n gencode_alignment_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

## Step 8 Pipeline Summary (MultiQC) -------------------------------------------------------------------------------------------
# Runs MultiQC on all of the files generated throughout the pipeline and collates them into a single quality assessment report.
# Activates the "multiqc" Conda environment which MultiQC runs within.
eval "$(conda shell.bash hook)"
conda activate multiqc
# Changes directories to /<path_to_file>/paired/results.
cd /<path_to_file>/paired/results/
# Runs multiqc in the current directory.
multiqc -n pipeline_multiqc_report.html .
# Deactivates the "multiqc" Conda environment.
conda deactivate

date
